﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Runtime.Serialization;

namespace HeuristicLab.Services.Optimization.ControllerService.Model {
  [DataContract]
  public enum ParameterType {
    [EnumMember]
    Decimal,
    [EnumMember]
    Integer,
    [EnumMember]
    Boolean,
    [EnumMember]
    Type,
    [EnumMember]
    DecimalMatrix,
    [EnumMember]
    DecimalVector,
    [EnumMember]
    Percent
  }

  [DataContract]
  public abstract class Value {    
  }

  [DataContract]
  public class DecimalValue : Value {
    [DataMember]
    public double Value { get; set; }
  }

  [DataContract]
  public class BoolValue : Value {
    [DataMember]
    public bool Value { get; set; }
  }

  [DataContract]
  [KnownType(typeof(BoolValue))]
  [KnownType(typeof(DecimalValue))]
  public class Parameter {
    [DataMember]
    public string Name { get; set; }
    
    // some kind of value type
    [DataMember]
    public ParameterType Type { get; set; }
    
    [DataMember]
    public Value Value { get; set; }

    /*
    //[DataMember]
    public object[] Choices { get; set; }*/
  }

  [DataContract]
  public class OptimizationScenario {
    [DataMember]
    public string Name { get; set; }

    private IList<Parameter> inputParameters = new List<Parameter>();

    [DataMember]
    public IList<Parameter> InputParameters {
      get { return inputParameters; }
      set { inputParameters = value; }
    }

    private IList<Parameter> algorithmParameters = new List<Parameter>();

    [DataMember]
    public IList<Parameter> AlgorithmParameters {
      get { return algorithmParameters; }
      set { algorithmParameters = value; }
    }
  }
}
